% Author: U. Zölzer
%
%--------------------------------------------------------------------------
% This source code is provided without any warranties as published in 
% DASP book 3rd edition, copyright Wiley & Sons 2022, available at 
% https://mydasp.com/dasp. It may be used for educational purposes and not 
% for commercial applications without further permission.
%--------------------------------------------------------------------------
% dasp_chap1_ex1_1
% March 28, 2022, Udo Zoelzer
% Exercise for Chapter "Introduction"
clf,clear;
%% Part 1 - Figure 1
% Sketch the impulse responses
t=[0 1 2 3 4 5 6 7];
a=[1 0 0 0 0 0 0 0];
b=[0 0 0 0 1 0 0 0];
c=[0.5 0.5 0 0 0 0 0 0];
d=[0.25 0.25 0.25 0.25 0 0 0 0];
stp=[1 1 1 1 1 1 1 1];
ex=(0.6).^t;

n=0:200;
cex=0.93.^n.*sin(n*2*pi/20);

% Plot the impulse responses using Matlab
figure(1)
subplot(321)
stem(t,a,'o','Linewidth',1), axis([-.6 6.5 -1.5 1.5]),
xlabel('n \rightarrow');grid;box ON;title('(a)');
ylabel('h(n)');

subplot(322)
stem(t,b,'o','Linewidth',1), axis([-.6 6.5 -1.5 1.5]),
xlabel('n \rightarrow');grid;box ON;title('(b) M=4')
ylabel('h(n)');

subplot(323)
stem(t,c,'o','Linewidth',1), axis([-.6 6.5 -0.25 1]),
xlabel('n \rightarrow');grid;box ON;title('(c)')
ylabel('h(n)');

subplot(324)
stem(t,d,'o','Linewidth',1), axis([-.6 6.5 -0.05 0.3]),
xlabel('n \rightarrow');grid;box ON;title('(d) M=4')
ylabel('h(n)');

subplot(325)
stem(t,ex.*stp,'o','Linewidth',1), axis([-.6 6.5 -1.5 1.5]),
xlabel('n \rightarrow');grid;box ON;title('(e)  a=0.6')
ylabel('h(n)');

subplot(326)
stem(n,cex,'.','Linewidth',1), axis([-.6 50 -1 1]),
xlabel('n \rightarrow');grid;box ON;title('(f)  a=0.93');
ylabel('h(n)');

print -depsc dasp_ex1_1

%% Figure 3
% dasp_chap1_ex1_1b
% Compute output y(n) for input x(n)=\delta(n)+ \delta(n-1)
x=[1 1];
y_a=conv(x,a);
y_b=conv(x,b);
y_c=conv(x,c);
y_d=conv(x,d);
y_e=conv(x,ex);
y_f=conv(x,cex);

%%
figure(2) 
subplot(321)
stem(n(1:length(a)+length(x)-1),y_a,'o','Linewidth',1), axis([-.6 6.5 -1.5 1.5]),
xlabel('n \rightarrow');grid;box ON;title('(a)');
ylabel('y(n)');

subplot(322)
stem(n(1:length(b)+length(x)-1),y_b,'o','Linewidth',1), axis([-.6 6.5 -1.5 1.5]),
xlabel('n \rightarrow');grid;box ON;title('(b) M=4')
ylabel('y(n)');

subplot(323)
stem(n(1:length(c)+length(x)-1),y_c,'o','Linewidth',1), axis([-.6 6.5 -0.25 1.1]),
xlabel('n \rightarrow');grid;box ON;title('(c)')
ylabel('y(n)');

subplot(324)
stem(n(1:length(d)+length(x)-1),y_d,'o','Linewidth',1), axis([-.6 6.5 -0.05 1.1]),
xlabel('n \rightarrow');grid;box ON;title('(d) M=4')
ylabel('y(n)');

subplot(325)
stem(n(1:length(ex)+length(x)-1),y_e,'o','Linewidth',1), axis([-.6 6.5 -1.5 2]),
xlabel('n \rightarrow');grid;box ON;title('(e)  a=0.6')
ylabel('y(n)');

subplot(326)
stem(n,y_f(n+1),'.','Linewidth',1), axis([-.6 50 -1 2]),
xlabel('n \rightarrow');grid;box ON;title('(f)  a=0.93');
ylabel('y(n)');

print -depsc dasp_ex1_2

%% Part 2 - Figure 3
% Unit pulse as input
% x=zeros(length(n),1);
% x(1)=1;
%

% M-tap input
x=zeros(length(n),1);
x(1:8)=1;
%

% Cosine input
% x=0.75*cos(2*pi*2/20*n);
%

% If impulse response is available you can use y=conv(x,h) 
% or alternatively y=filter(h,1,x)
% If difference equation is available use filter(b,a,x) command
y_2a=filter(cex,1, x);
y_2b=filter(1, [1 -0.5], x);
y_2c=filter(1, [1 -1], x);
y_2d=filter([1 -1], 1, x);

%% 
figure(3)

subplot(321)
stem(0:length(y_2a)-1,y_2a,'.','Linewidth',1), axis([-.6 50 -max(abs(y_2a))-0.5 +max(abs(y_2a))+0.5]),
xlabel('n \rightarrow');grid;box ON;title('(a)');
ylabel('y(n)');

subplot(322)
stem(0:length(y_2b)-1,y_2b,'.','Linewidth',1), axis([-.6 50 -max(abs(y_2b))-0.5 +max(abs(y_2b))+0.5]),
xlabel('n \rightarrow');grid;box ON;title('(b)');
ylabel('y(n)');

subplot(323)
stem(0:length(y_2c)-1,y_2c,'.','Linewidth',1), axis([-.6 50 -max(abs(y_2c))-0.5 +max(abs(y_2c))+0.5]),
xlabel('n \rightarrow');grid;box ON;title('(c)');
ylabel('y(n)');

subplot(324)
stem(0:length(y_2d)-1,y_2d,'.','Linewidth',1), axis([-.6 50 -max(abs(y_2d))-0.5 +max(abs(y_2d))+0.5]),
xlabel('n \rightarrow');grid;box ON;title('(d)');
ylabel('y(n)');

% Unit pulse as input
% print -depsc dasp_ex1_3
% M-tap input
print -depsc dasp_ex1_4