% Author: U. Zölzer
%
%--------------------------------------------------------------------------
% This source code is provided without any warranties as published in 
% DASP book 3rd edition, copyright Wiley & Sons 2022, available at 
% https://mydasp.com/dasp. It may be used for educational purposes and not 
% for commercial applications without further permission.
%--------------------------------------------------------------------------
% dasp_ex1_4_2
% March 28, 2022, Udo Zoelzer
% Exercise for Chapter "Introduction"
h_ref = firpm(11, [0 .4 .6 1], [1 1 0 0]);
N = length(h_ref); % filter length
p=N-1;             % prediction order
figure(1)
clf
subplot(211)
stem(0:N-1, h_ref)
title('Reference impulse response')
ylabel('h_{ref}(n)')
subplot(212)
zplane(h_ref)
print -depsc dasp_ex1_4_sysid_a

Nx=1000;
mu=1.4;

X = randn(1, Nx); % noise
X = X/max(abs(X));
D = filter(h_ref, 1, X); % desired signal
x_buff=zeros(1,N); % filter input
b=zeros(1,N);  % direct FIR coeffs
E=zeros(1,Nx); % error
Y=zeros(1,Nx); % output
CoefDiff=zeros(1,Nx); % coeff difference
for n=1:Nx
%shift filter input states:
x_buff = [X(n), x_buff(1:N-1)]; %newest input sample first
Y(n) = x_buff*b'; % filter result
E(n) = D(n) - Y(n); % prediction error
for i=1:N % Update of coefficients with LMS
b(i) = b(i) + mu*x_buff(i)*E(n);
end
CoefDiff(n) = 10*log10( sum(b - h_ref).^2 );
end

figure(2)
subplot(511),plot(X);ylabel('x(n)');
subplot(512),plot(D);ylabel('d(n)');
subplot(513),plot(Y);ylabel('y(n)');
subplot(514),plot(E);ylabel('e(n)');
subplot(515),plot(CoefDiff);ylabel('CoefDiff(n) in dB');
xlabel('n \rightarrow');

%print -depsc dasp_ex1_4_sysid_b